from ..db import model as m

def insert_study(record: m.SraStudy):
    def insert_attributes():
        def insert_tags():
    def insert_identifiers():
        def insert_namespace()
    # study_accession (FK to study)
    # identifier(text)
    # namespace_id (
    # TODO: need to convert namespace to use id
    
    def insert_pubmed_ids()
    def 
